/**
 * BBPhaseTest.java
 * Created: Feb 18, 2006
 */
package tests.jmarkov;

import static tests.jmarkov.Utils.assertArrayEquals;
import jmarkov.basic.exceptions.NotUnichainException;
import jphase.ContPhaseVar;
import jphase.DenseContPhaseVar;
import junit.framework.TestCase;
import examples.jmarkov.BBPhase;
import examples.jmarkov.BucketBrigades;

/**
 * @author Germn Riao. Universidad de los Andes. (C) 2006
 * 
 */
public class BBPhaseTest extends TestCase {

    BBPhase bbp = null;

    /**
     * @param args Not used
     */
    public static void main(String[] args) {
        junit.swingui.TestRunner.run(BBPhaseTest.class);
    }

    /*
     * @see TestCase#setUp()
     */
    @Override
    protected void setUp() throws Exception {
        int Op = 2;
        int M = 2;
        DenseContPhaseVar v = DenseContPhaseVar.Erlang(1, 2);
        DenseContPhaseVar[] distMaq = { v, v };
        double[][] vels = { { 1, 1 }, { 2, 3 }, };
        bbp = new BBPhase(Op, M, distMaq, vels);
        bbp.setDebugLevel(0);
    }

    /*
     * @see TestCase#tearDown()
     */
    @Override
    protected void tearDown() throws Exception {
        bbp = null;
        super.tearDown();
    }

    /**
     * Test method for 'jmarkov.SimpleMarkovProcess.getMOPsAvg()'
     * @throws NotUnichainException 
     */
    public void testGetMOPsAvg() throws NotUnichainException {
        double mops[] = bbp.getMOPsAvg();
        double expMops[] = new double[] { 0.43346007631073086,
                1.000000001896173, 1.4334600782069038, 0.5665399255854421, 0.0 };
        assertArrayEquals("Mops are different", expMops, mops, 1e-5);
    }

    /**
     * Test method for 'jmarkov.SimpleMarkovProcess.getEventsRates()'
     * @throws NotUnichainException 
     */
    public void testGetEventsRates() throws NotUnichainException {
        double rates[] = bbp.getEventsRates();
        double expRates[] = new double[] { 0.31939163503624357,
                0.11406844127448725, 1.1406844125743791, 1.3460076062767956 };
        assertArrayEquals("Event Rates are different", expRates, rates, 1e-5);
    }

    /**
     * @throws NotUnichainException 
     * 
     */
    public void testThruputRate() throws NotUnichainException {
        double val = bbp.getResetRate();
        assertEquals("Error in Throughput ", 0.7300380220716589, val, 1e-5);
    }

    /**
     * test for expo servers
     * @throws NotUnichainException 
     */

    public void testExpo() throws NotUnichainException {
        double mu = 1.0;// service rate
        for (int M = 1; M <= 5; M += 2) {
            for (int N = 1; N <= 3; N += 2) {// Operators
                ContPhaseVar times[] = new ContPhaseVar[M];
                double vels[][] = new double[N][M];
                for (int n = 0; n < N; n++) {
                    for (int m = 0; m < M; m++) {
                        vels[n][m] = 1.0 / (n + 1);
                    }
                }
                ContPhaseVar v = DenseContPhaseVar.expo(mu);
                for (int m = 0; m < M; m++)
                    times[m] = v;
                bbp = new BBPhase(N, M, times, vels);
                bbp.setDebugLevel(0);
                bbp.setMaxStates(2000);

                BucketBrigades bb = new BucketBrigades(N, M, vels);
                bb.setDebugLevel(0);
                bb.setMaxStates(2000);
                double expTh = bb.getEventRate(M - 1);

                double val = bbp.getResetRate();
                bbp.debug(0, "M=" + M + ", N= " + N + ", States = "
                        + bbp.getNumStates());
                assertEquals("Error in Thruput for N=" + N + ", M=" + M, expTh,
                        val, 1e-5);
            }
        }
    }

    /**
     * test for identical expo servers
     * @throws NotUnichainException 
     */
    public void testEqualExpo() throws NotUnichainException {
        double mu = 1.0;// service rate
        for (int M = 1; M <= 7; M += 2) {
            for (int N = 1; N <= 5; N += 2) {// Operators
                ContPhaseVar times[] = new ContPhaseVar[M];
                double vels[][] = new double[N][M];
                for (int n = 0; n < N; n++) {
                    for (int m = 0; m < M; m++) {
                        vels[n][m] = 1.0;
                    }
                }
                ContPhaseVar v = DenseContPhaseVar.expo(mu);
                for (int m = 0; m < M; m++)
                    times[m] = v;
                double expTh = N * mu / (M + N - 1);
                bbp = new BBPhase(N, M, times, vels);
                bbp.setDebugLevel(0);
                bbp.setMaxStates(2000);
                double val = bbp.getResetRate();
                bbp.debug(0, "M=" + M + ", N= " + N + ", States = "
                        + bbp.getNumStates());
                assertEquals("Error in Thruput ", expTh, val, 1e-10);
            }
        }
    }

}
