package tests.jmdp;

import jmarkov.jmdp.solvers.PolicyIterationSolver;
import jmarkov.jmdp.solvers.RelativeValueIterationSolver;
import jmarkov.jmdp.solvers.Solver;
import jmarkov.jmdp.solvers.ValueIterationSolver;
import junit.framework.TestCase;
import examples.jmdp.InfStochasticDemand;
import examples.jmdp.InvLevel;
import examples.jmdp.Order;

//DATA:
//N : 15 ! Max inventory
//B : 5 ! Max backorders
//mean: 4 ! mean demand
//K: 6 !Rig capacity
//p : 22 ! sale price
//c : 20 ! aquisition cost
//int : .30 ! year interest 
//b : 1 ! discount per backorder
//A : 0.5 !Fixed cost per rig
//Status    : Se hall un ptimo.
//Cost: 580.482
//SOLUTION:
//States ={-5,-4,-3,-2,-1,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15}
//Computing parameters...
//Writing Probabilities and Costs to CSV files.
//Solving...
//x has 216 components
//Problem Status    : Optimal found.
//Cost:     0.8242
//Optimal Policy:
//In state '-5' (prob= 0.002041) take action '12'
//In state '-4' (prob= 0.003554) take action '12'
//In state '-3' (prob= 0.008503) take action '12'
//In state '-2' (prob= 0.018387) take action '12'
//In state '-1' (prob= 0.035637) take action '6'
//In state '0' (prob= 0.061356) take action '6'
//In state '1' (prob= 0.093034) take action '6'
//In state '2' (prob= 0.123404) take action '6'
//In state '3' (prob= 0.142892) take action '6'
//In state '4' (prob= 0.145209) take action '6'
//In state '5' (prob= 0.130904) take action '0'
//In state '6' (prob= 0.104836) take action '0'
//In state '7' (prob= 0.072250) take action '0'
//In state '8' (prob= 0.039680) take action '0'
//In state '9' (prob= 0.015261) take action '0'
//In state '10' (prob= 0.003052) take action '0'
//
//
//REPORT
//State                   -5        -4        -3        -2        -1         0         1         2         3         4         5         6         7         8         9        10        11        12        13        14        15
//Prob                0.002     0.004     0.009     0.018     0.036     0.061     0.093     0.123     0.143     0.145     0.131     0.105     0.072     0.040     0.015     0.003     0.000     0.000     0.000     0.000     0.000
//Init Inv.              -5        -4        -3        -2        -1         0         1         2         3         4         5         6         7         8         9        10        11        12        13        14        15
//Buy                    12        12        12        12         6         6         6         6         6         6         0         0         0         0         0         0         0         0         0         0         0
//Availabl.               7         8         9        10         5         6         7         8         9        10         5         6         7         8         9        10        11        12        13        14        15
//Exp. Sales          4.000     4.000     4.000     4.000     3.996     3.999     4.000     4.000     4.000     4.000     3.996     3.999     4.000     4.000     4.000     4.000     4.000     4.000     4.000     4.000     4.000
//Lost Sales          0.000     0.000     0.000     0.000     0.004     0.001     0.000     0.000     0.000     0.000     0.004     0.001     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000
//Sales Income       87.992    87.998    87.999    88.000    87.909    87.972    87.992    87.998    87.999    88.000    87.909    87.972    87.992    87.998    87.999    88.000    88.000    88.000    88.000    88.000    88.000
//Truck Cost          1.000     1.000     1.000     1.000     0.500     0.500     0.500     0.500     0.500     0.500     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000
//Order Cost        240.000   240.000   240.000   240.000   120.000   120.000   120.000   120.000   120.000   120.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000
//Holding Cost        0.000     0.000     0.000     0.000     0.000     0.000     0.101     0.202     0.303     0.405     0.506     0.607     0.708     0.809     0.910     1.012     1.113     1.214     1.315     1.416     1.517
//Backorder Cost      5.000     4.000     3.000     2.000     1.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000     0.000
//Tot. Oprt. Cost     6.001     5.000     4.000     3.000     1.508     0.503     0.602     0.703     0.804     0.905     0.514     0.610     0.709     0.810     0.911     1.012     1.113     1.214     1.315     1.416     1.517
//Net Income       -158.008  -157.002  -156.001  -155.000   -33.591   -32.528   -32.609   -32.705   -32.804   -32.905    87.403    87.365    87.284    87.188    87.089    86.988    86.887    86.786    86.685    86.584    86.483

/**
 * 
 * @author Andres Sramiento, German Riano. Universidad de los Andes. (C) 2006
 * 
 */

public class InfStochasticDemandTest extends TestCase {

    InfStochasticDemand prob = null;

    @Override
    protected void setUp() throws Exception {
        int maxInventory = 10;
        int maxBackorders = 5;
        int truckSize = 6;
        int K = 500;
        double b = 1000;
        double cost = 20000;
        double h0 = 0.0, // non-financial cost
        intRate = Math.pow(1.3, 1.0 / 52)-1.0;
        double theta = 4;
        double price = 22000;
        prob = new InfStochasticDemand(maxInventory, maxBackorders, truckSize,
                K, b, price, cost, h0, intRate, theta, true);
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        prob = null;
    }

    // public void testMain(){
    // InfStochasticDemand.main(new String[]{});
    // }

    /**
     * @throws Exception
     */
    public void testPrintMatrices() throws Exception{
        prob.printMatrices();
    }
    
    
    /**
     * @throws Exception
     */
    public void testGetOptimalValueFunctionRelValueIteration()
            throws Exception {
        InvLevel initialState = new InvLevel(0);
        double vf_cero = -8242; //1762.6502;// -16840.54478;

        Solver<InvLevel, Order> solv = new RelativeValueIterationSolver<InvLevel, Order>(
                prob);
        prob.setSolver(solv);
        double vf = prob.getOptimalValueFunction().get(initialState);
        System.out.println("Realtive Value Iteration : "
                + solv.getProcessTime() + " milliseconds");
        solv = null;
        assertEquals("Value Function not equal",  vf_cero, vf, 1.0E-3);
    }
    /**
     * @throws Exception
     */
    public void testGetOptimalValueFunctionValueIterationNoError()
            throws Exception {
        InvLevel initialState = new InvLevel(0);
        double vf_cero = 1762.6502;// -16840.54478;

        ValueIterationSolver<InvLevel, Order> solv = new ValueIterationSolver<InvLevel, Order>(
                prob, 0.9);
        prob.setSolver(solv);
        double vf = prob.getOptimalValueFunction().get(initialState);
//        System.out.println("Value Iteration No error bounds: "
//                + solv.getProcessTime() + " milliseconds");
        solv = null;
        assertEquals("Value Function not equal",  vf_cero, vf, 1.0E-3);
    }

    /**
     * @throws Exception
     */
    public void testGetOptimalValueFunctionValueIterationError()
            throws Exception {
        InvLevel initialState = new InvLevel(0);
        double vf_cero = 1762.6502;

        ValueIterationSolver<InvLevel, Order> solv = new ValueIterationSolver<InvLevel, Order>(
                prob, 0.9);
        solv.useErrorBounds(true);
        prob.setSolver(solv);
        double vf = prob.getOptimalValueFunction().get(initialState);
//        System.out.println("Value Iteration with error bounds: "
//                + solv.getProcessTime() + " milliseconds");
        solv = null;

        assertEquals("Value Function not equal", vf_cero, vf, 1.0E-3);
    }

    /**
         * @throws Exception
         */
        public void testGetOptimalValueFunctionPolicyIteration() throws Exception {
            InvLevel initialState = new InvLevel(0);
            double vf_cero = 1762.6502;
    
            PolicyIterationSolver<InvLevel, Order> solv = new PolicyIterationSolver<InvLevel, Order>(
                    prob, 0.9);
            prob.setSolver(solv);
            prob.setDebugLevel(4);
            prob.solve();
            double vf = prob.getOptimalValueFunction().get(initialState);
    //        System.out.println("Policy Iteration: " + solv.getProcessTime()
    //                + " milliseconds");
            solv = null;
    
            assertEquals("Value Function not equal", vf_cero, vf, 1.0E-3);
        }

    /**
         * @throws Exception
         */
        public void testGetOptimalValueFunctionModifiedPolicy() throws Exception {
            InvLevel initialState = new InvLevel(0);
            double vf_cero = 1762.6502;
    
            PolicyIterationSolver<InvLevel, Order> solv = new PolicyIterationSolver<InvLevel, Order>(
                    prob, 0.9);
            solv.setModifiedPolicy(true);
            double vf = prob.getOptimalValueFunction().get(initialState);
    //        System.out.println("Modified Policy Iteration: "
    //                + solv.getProcessTime() + " milliseconds");
            solv = null;
    
            assertEquals("Value Function not equal", vf_cero, vf, 1.0E-3);
        }

    /**
     * @param args
     */
    public static void main(String[] args) {
        junit.swingui.TestRunner.run(InfStochasticDemandTest.class);
    }
}
