/**
 * testSolvers.java
 * Created: 30/03/2006
 */
package tests.jmdp;

import java.io.File;
import java.io.IOException;
import java.util.Formatter;

import jmarkov.basic.exceptions.SolverException;
import jmarkov.jmdp.solvers.LPBCLAverageSolver;
import jmarkov.jmdp.solvers.LPBCLDiscountedSolver;
import jmarkov.jmdp.solvers.LPSolver;
import jmarkov.jmdp.solvers.MpsLpAverageSolver;
import jmarkov.jmdp.solvers.MpsLpDiscountedSolver;
import jmarkov.jmdp.solvers.PolicyIterationSolver;
import jmarkov.jmdp.solvers.RelativeValueIterationSolver;
import jmarkov.jmdp.solvers.Solver;
import jmarkov.jmdp.solvers.ValueIterationSolver;
import examples.jmdp.ControlProdNonEvents;
import examples.jmdp.InvLevel;
import examples.jmdp.Order;
import examples.jmdp.solvers.MPSQsOptAverageSolver;
import examples.jmdp.solvers.MPSQsOptDiscountedSolver;
import examples.jmdp.solvers.MPSXpressAverage;
import examples.jmdp.solvers.MPSXpressDiscounted;

/**
 * @author Diego Bello, German Riano. Universidad de los Andes. (C)
 *         2006
 */
public class SolversBenchmarkTest {

    /**
     * @param a
     */
    public static void main(String a[]) {
        Formatter output;
        // int M = 4;// Capacity
        double K = 500;// Fixed cost per Order
        double cost = 400; // variable cost
        double price = 1000; // variable cost
        double holdingCost = 80; // holding cost per computer per
        // period.
        double interestRate = 0.10;
        double demandMean = 4; // mean of the Poisson demand per
        // stage
        String folder = "tests\\jmdp\\DiscTest";
        String fileName = "SolversBenchmarkTest.csv";
        output = openFile(folder, fileName);

        for (int M = 4; M < 1025; M = 2 * M) {

            ControlProdNonEvents prob = new ControlProdNonEvents(M, K, cost,
                    price, holdingCost, interestRate, demandMean);
            prob.setDebugLevel(0);
            prob.getAllStates();

            Solver<InvLevel, Order> solver1 = new PolicyIterationSolver<InvLevel, Order>(
                    prob, interestRate, true);
            Solver<InvLevel, Order> solver2 = new ValueIterationSolver<InvLevel, Order>(
                    prob, interestRate);
            LPBCLDiscountedSolver<InvLevel, Order> solver3 = new LPBCLDiscountedSolver<InvLevel, Order>(
                    prob, interestRate);
            MPSXpressDiscounted<InvLevel, Order> solver4 = new MPSXpressDiscounted<InvLevel, Order>(
                    prob, interestRate, folder, "MDPDisc.mps");
            MpsLpDiscountedSolver<InvLevel, Order> solver5 = new MPSQsOptDiscountedSolver<InvLevel, Order>(
                    prob, interestRate, folder, "MDPDisc.mps");
            // Average solvers:
            LPBCLAverageSolver<InvLevel, Order> solver6 = new LPBCLAverageSolver<InvLevel, Order>(
                    prob);
            MPSXpressAverage<InvLevel, Order> solver7 = new MPSXpressAverage<InvLevel, Order>(
                    prob, folder, "MDPDisc.mps");
            RelativeValueIterationSolver<InvLevel, Order> solver8 = new RelativeValueIterationSolver<InvLevel, Order>(
                    prob);
            MpsLpAverageSolver<InvLevel, Order> solver9 = new MPSQsOptAverageSolver<InvLevel, Order>(
                    prob, folder, "MDPDisc.mps");
            Solver solvers[] = { solver1, solver2, solver3, solver4, solver5,
                    solver6, solver7, solver8, solver9 };
            printNames(solvers, output, M);
            for (int loops = 0; loops < 20; loops++) {
                for (Solver solver : solvers) {
                    solveProblem(solver, output);
                }
                output.format("\n");
            }
            output.flush();

        }
        output.format("\n\n");

        if (output != null)
            output.close();
    }

    private static void printNames(Solver[] solvers, Formatter frm, int M) {
        System.out.printf("%tT States : " + M + "\n", System
                .currentTimeMillis());
        frm.format("%tT : States : " + M + "\n", System.currentTimeMillis());
        for (Solver solver : solvers) {
            String label = solver.label();
            if (solver instanceof LPSolver)
                frm.format(label + "_B," + label + "_LP," + label + "_R,"
                        + label + "_T,");
            else
                frm.format(label + "_T,");
        }
        frm.format("\n");
        frm.flush();
    }

    private static void solveProblem(Solver solver, Formatter frm) {

        // prob.setSolver(solver9);
        try {
            solver.solve();
        } catch (SolverException e) {
            frm.format("Exception:" + e);
            System.err.println(e);
        }
        if (solver instanceof LPSolver) {
            LPSolver sol = (LPSolver) solver;
            long solverBuildTime = sol.getBuildTime();
            long solverLPSolveTime = sol.getLpSolveTime();
            long solverSolBuildTime = sol.getSolBuildTime();
            frm.format("%10d, %10d, %10d,", solverBuildTime, solverLPSolveTime,
                    solverSolBuildTime);
        }
        long solverProcessTime = solver.getProcessTime();
        frm.format("%10d,", solverProcessTime);
        System.gc();
    }

    /**
     * @param output
     * @param folder
     * @param fileName
     * @return The openned file.
     * @throws SolverException
     */
    private static Formatter openFile(String folder, String fileName)
            throws RuntimeException {
        Formatter output;
        try {
            output = new Formatter(new File(folder, fileName));
        } // end try
        catch (SecurityException securityException) {
            throw new RuntimeException(
                    "You do not have write access to  file: " + fileName);
        } // end catch
        catch (IOException fileNotFoundException) {
            throw new RuntimeException("Error creating file: " + fileName);
        } // end catch
        return output;
    }

}

// prob.setSolver(solver1);
// prob.solve();
// long solver1BuildTime = solver1.getBuildTime();
// long solver1LPSolveTime = solver1.getLpSolveTime();
// long solver1SolBuildTime = solver1.getSolBuildTime();
// long solver1ProcessTime = solver1.getProcessTime();
// System.gc();
//
// prob.setSolver(solver2);
// prob.solve();
// long solver2BuildTime = solver2.getBuildTime();
// long solver2LPSolveTime = solver2.getLpSolveTime();
// long solver2SolBuildTime = solver2.getSolBuildTime();
// long solver2ProcessTime = solver2.getProcessTime();
// System.gc();
//
// prob.setSolver(solver3);
// prob.solve();
// long solver3ProcessTime = solver3.getProcessTime();
// System.gc();
//
// prob.setSolver(solver4);
// prob.solve();
// long solver4ProcessTime = solver4.getProcessTime();
// System.gc();
//
// prob.setSolver(solver5);
// prob.solve();
// long solver5BuildTime = solver5.getBuildTime();
// long solver5LPSolveTime = solver5.getLpSolveTime();
// long solver5SolBuildTime = solver5.getSolBuildTime();
// long solver5ProcessTime = solver5.getProcessTime();
// System.gc();
//
// prob.setSolver(solver6);
// prob.solve();
// long solver6BuildTime = solver6.getBuildTime();
// long solver6LPSolveTime = solver6.getLpSolveTime();
// long solver6SolBuildTime = solver6.getSolBuildTime();
// long solver6ProcessTime = solver6.getProcessTime();
// System.gc();
//
// prob.setSolver(solver7);
// prob.solve();
// long solver7BuildTime = solver7.getBuildTime();
// long solver7LPSolveTime = solver7.getLpSolveTime();
// long solver7SolBuildTime = solver7.getSolBuildTime();
// long solver7ProcessTime = solver7.getProcessTime();
// System.gc();
//
// prob.setSolver(solver8);
// prob.solve();
// long solver8ProcessTime = solver8.getProcessTime();
// System.gc();
//
// prob.setSolver(solver9);
// prob.solve();
// long solver9BuildTime = solver9.getBuildTime();
// long solver9LPSolveTime = solver9.getLpSolveTime();
// long solver9SolBuildTime = solver9.getSolBuildTime();
// long solver9ProcessTime = solver9.getProcessTime();
// System.gc();
//
// output
// .format(
// ",%10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d,
// %10d, %10d, %10d, %10d, "
// + "%10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d, %10d,
// %10d, %10d\n",
// solver3ProcessTime, solver4ProcessTime,
// solver1BuildTime, solver1LPSolveTime,
// solver1SolBuildTime, solver1ProcessTime,
// solver2BuildTime, solver2LPSolveTime,
// solver2SolBuildTime, solver2ProcessTime,
// solver5BuildTime, solver5LPSolveTime,
// solver5SolBuildTime, solver5ProcessTime,
// solver6BuildTime, solver6LPSolveTime,
// solver6SolBuildTime, solver6ProcessTime,
// solver7BuildTime, solver7LPSolveTime,
// solver7SolBuildTime, solver7ProcessTime,
// solver8ProcessTime,//
// solver9BuildTime, solver9LPSolveTime,
// solver9SolBuildTime, solver9ProcessTime
//
// );
