/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

/**
 * Vector consisting of sub-vectors
 */
public interface BlockAccessVector extends Vector {

	/**
	 * Returns number of blocks
	 */
	int numBlocks();

	/**
	 * Returns the vector in block i
	 */
	Vector getBlock(int i);

	/**
	 * Returns the indices of block i
	 */
	int[] getBlockIndices(int i);

	/**
	 * Sets block i to the given vector with indices
	 */
	void setBlock(int i, int[] index, Vector x);

	/**
	 * Returns an array of the same size as this vector. Each element is zeroed.
	 * The purpose of this method is to avoid excessive memory allocation and
	 * de-allocation. Note that this is not thread safe
	 */
	double[] getArray();

}
