/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

/**
 * Block matrix
 */
public class BlockMatrix
	extends AbstractMatrix
	implements Matrix, BlockAccessMatrix {

	/**
	 * Sub-matrices (blocks)
	 */
	private Matrix[] A;

	/**
	 * Row-indices for each block
	 */
	private int[][] row;

	/**
	 * Column-indices for each block
	 */
	private int[][] column;

	/**
	 * Locks for thread-safety
	 */
	private Lock[] lock;

	/**
	 * Constructor for BlockMatrix.
	 */
	public BlockMatrix(int numRows, int numColumns, int numBlocks) {
		super(numRows, numColumns);
		A = new Matrix[numBlocks];
		row = new int[numBlocks][];
		column = new int[numBlocks][];
		lock = Lock.createLocks(numBlocks);
	}

	/**
	 * @see jmp.BlockAccessMatrix#numBlocks()
	 */
	public int numBlocks() {
		return A.length;
	}

	/**
	 * @see jmp.BlockAccessMatrix#getBlock(int)
	 */
	public Matrix getBlock(int i) {
		return A[i];
	}

	/**
	 * @see jmp.BlockAccessMatrix#getBlockRowIndices(int)
	 */
	public int[] getBlockRowIndices(int i) {
		return row[i];
	}

	/**
	 * @see jmp.BlockAccessMatrix#getBlockColumnIndices(int)
	 */
	public int[] getBlockColumnIndices(int i) {
		return column[i];
	}

	/**
	 * @see jmp.BlockAccessMatrix#setBlock(int, int[], int[], jmp.Matrix)
	 */
	public void setBlock(int i, int[] row, int[] column, Matrix A) {
		synchronized (lock[i]) {
			this.row[i] = row;
			this.column[i] = column;
			this.A[i] = A;
		}
	}

}
