/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

import java.util.Arrays;

/**
 * Block vector
 */
public class BlockVector
	extends AbstractVector
	implements Vector, BlockAccessVector {

	/**
	 * Sub-vectors (blocks)
	 */
	private Vector[] x;

	/**
	 * Indices for each block
	 */
	private int[][] index;

	/**
	 * Locks for thread-safety
	 */
	private Lock[] lock;

	/**
	 * Dense representation (only valid after assemble())
	 */
	private double[] dense;

	/**
	 * Constructor for BlockVector.
	 */
	public BlockVector(int size, int numBlocks) {
		super(size);
		x = new Vector[numBlocks];
		index = new int[numBlocks][];
		lock = Lock.createLocks(numBlocks);
		dense = new double[size];
	}

	/**
	 * @see jmp.BlockAccessVector#numBlocks()
	 */
	public int numBlocks() {
		return x.length;
	}

	/**
	 * @see jmp.BlockAccessVector#getBlock(int)
	 */
	public Vector getBlock(int i) {
		return x[i];
	}

	/**
	 * @see jmp.BlockAccessVector#getBlockIndices(int)
	 */
	public int[] getBlockIndices(int i) {
		return index[i];
	}

	/**
	 * @see jmp.BlockAccessVector#setBlock(int, int[], jmp.Vector)
	 */
	public void setBlock(int i, int[] index, Vector x) {
		synchronized (lock[i]) {
			this.index[i] = index;
			this.x[i] = x;
		}
	}

	/**
	 * @see jmp.BlockAccessVector#getArray()
	 */
	public double[] getArray() {
		Arrays.fill(dense, 0.);
		return dense;
	}

}
