/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

/**
 * Vector with elemental access operations. The block-wise operations will
 * typically be faster than the elementwise operations, and the extractions
 * methods present here are not likely to be as fast as the custom accessors
 * the vector may implement.
 */
public interface ElementalAccessVector extends Vector {

	/**
	 * x(index) += value
	 */
	void addValue(int index, double value);

	/**
	 * x(index) = value
	 */
	void setValue(int index, double value);

	/**
	 * Returns x(index)
	 */
	double getValue(int index);

	/**
	 * x(index) += values, blockwise
	 */
	void addValues(int[] index, double[] values);

	/**
	 * x(index) = values, blockwise
	 */
	void setValues(int[] index, double[] values);

	/**
	 * Returns the block x(index)
	 */
	double[] getValues(int[] index);

}
