/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

/**
 * An index/data pairing with integer indices and double-precision data.
 * Note that the length of the indices and the data need not be equal.
 * Furthermore, the indices must be kept sorted in most cases.
 */
public class IntDoubleVectorPair {

	/**
	 * Integer indices
	 */
	public int[] index;

	/**
	 * Double-precision data
	 */
	public double[] data;

	/**
	 * Constructor for IntDoubleVectorPair.
	 * @param index Indices
	 * @param data Data
	 */
	public IntDoubleVectorPair(int[] index, double[] data) {
		this.index = index;
		this.data = data;
	}

	/**
	 * Deep copy of the object
	 */
	public IntDoubleVectorPair copy() {
		int[] newIndex = new int[index.length];
		double[] newData = new double[data.length];
		System.arraycopy(index, 0, newIndex, 0, index.length);
		System.arraycopy(data, 0, newData, 0, data.length);
		return new IntDoubleVectorPair(newIndex, newData);
	}

	/**
	 * Copy of the data between index[i] and index[i+1]
	 */
	public double[] copy(int i) {
		double[] copy = new double[index[i + 1] - index[i]];
		System.arraycopy(data, index[i], copy, 0, copy.length);
		return copy;
	}

}
