/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

/**
 * Dual indices and data paired together. Used by some of the faster matrices.
 * major contains the index for each data-entry, while minor is either uses as
 * row- or column-delimiters. Within two minor-indices, the major-indices
 * must be sorted for most applications.
 */
public class IntIntDoubleVectorTriple {

	/**
	 * Major indices, as long as data
	 */
	public int[] major;

	/**
	 * Minor indices, shorter than data
	 */
	public int[] minor;

	/**
	 * Double-precision data
	 */
	public double[] data;

	/**
	 * Constructor for IntIntDoubleVectorTriple.
	 */
	public IntIntDoubleVectorTriple(int[] major, int[] minor, double[] data) {
		this.major = major;
		this.minor = minor;
		this.data = data;
	}

	/**
	 * Deep copy of the object
	 */
	public IntIntDoubleVectorTriple copy() {
		int[] newMajor = new int[major.length],
			newMinor = new int[minor.length];
		double[] newData = new double[data.length];
		System.arraycopy(major, 0, newMajor, 0, major.length);
		System.arraycopy(minor, 0, newMinor, 0, minor.length);
		System.arraycopy(data, 0, newData, 0, data.length);
		return new IntIntDoubleVectorTriple(newMajor, newMinor, newData);
	}

	/**
	 * Copy of the index/data pair between minor[i] and minor[i+1]
	 */
	public IntDoubleVectorPair copy(int i) {
		int length = minor[i + 1] - minor[i];
		int[] index = new int[length];
		double[] newData = new double[length];
		System.arraycopy(major, minor[i], index, 0, length);
		System.arraycopy(data, minor[i], newData, 0, length);
		return new IntDoubleVectorPair(index, newData);
	}

}
