/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

/**
 * Matrix with external storage and custom matrix/vector product
 */
public interface ShellMatrix extends Matrix {

	/**
	 * <code>z = alpha*A*x + beta*y</code>. x can not be the same as y or z.
	 * @return z
	 */
	Vector multAdd(double alpha, Vector x, double beta, Vector y, Vector z);

	/**
	 * <code>z = alpha*A<sup>T</sup>*x + beta*y</code>. x can not be the same as
	 * y or z.
	 * @return z
	 */
	Vector transMultAdd(
		double alpha,
		Vector x,
		double beta,
		Vector y,
		Vector z);

}
