/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp;

import EDU.oswego.cs.dl.util.concurrent.FJTaskRunnerGroup;

/**
 * Contains default settings. Users should only read the fields, not write to
 * the them. To modify the fields, use the method
 * {@link #setNumThreads(int) setNumThreads}.
 * @see jmp.util.ParallelWorker
 */
public class State {

	/**
	 * No need to instantiate this
	 */
	private State() {
	}

	/**
	 * Thread executor and queue
	 */
	public static FJTaskRunnerGroup runner;

	/**
	 * The number of threads to use in operations. Set equal the number of
	 * processors to use
	 */
	public static int numThreads;

	/**
	 * Algebraic operations. Chose desired implementation (sequential, parallel)
	 */
	public static BLAS BLAS;

	/**
	 * Default settings are sequential operations
	 */
	static {
		setNumThreads(1);
	}

	/**
	 * Sets number of threads to use, and updates the BLAS
	 */
	public static void setNumThreads(int numThreads) {
		State.numThreads = numThreads;
		runner = new FJTaskRunnerGroup(numThreads);

		// Use parallel operations only with sufficient number of threads
		if (numThreads > 1)
			BLAS = new ParallelBLAS();
		else
			BLAS = new SequentialBLAS();
	}

}
