/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;

import jmp.Matrix;
import jmp.Vector;

/**
 * Reads serialized matrices and vectors
 * <p>
 * All objects read have the same datatype as when they were written.
 * </p>
 */
public class SerializedReader
	extends ObjectInputStream
	implements MatrixVectorReader {

	/**
	 * Constructor for SerializedReader.
	 */
	public SerializedReader(InputStream in) throws IOException {
		super(in);
	}

	/**
	 * @see jmp.io.MatrixVectorReader#readMatrix()
	 */
	public Matrix readMatrix() throws IOException {
		try {
			return (Matrix) readObject();
		} catch(ClassNotFoundException e) {
			throw new IOException();
		}
	}

	/**
	 * @see jmp.io.MatrixVectorReader#readVector()
	 */
	public Vector readVector() throws IOException {
		try {
			return (Vector) readObject();
		} catch(ClassNotFoundException e) {
			throw new IOException();
		}
	}

}
