/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.eig;

import java.util.List;

import jmp.Vector;

/**
 * Registers iteration information into an array
 */
public class ArrayEigenvalueIterationMonitor
	implements EigenvalueIterationMonitor {

	/**
	 * Contains all the residuals
	 */
	private List array;

	/**
	 * Constructor for ArrayEigenvalueIterationMonitor.
	 */
	public ArrayEigenvalueIterationMonitor() {
		array = new java.util.Vector();
	}

	/**
	 * Retrives all the residuals in an array. Array index is the iteration
	 * number.
	 * @return Double array of residuals
	 */
	public double[][] getArray() {
		Object[] arr = array.toArray();
		double[][] ret = new double[arr.length][];
		for (int i = 0; i < arr.length; ++i)
			ret[i] = (double[]) arr[i];
		return ret;
	}

	/**
	 * @see jmp.iter.eig.EigenvalueIterationMonitor#monitor(double[], double[], jmp.Vector[], int)
	 */
	public void monitor(double[] r, double[] eig, Vector[] x, int i) {
		monitor(r, i);
	}

	/**
	 * @see jmp.iter.eig.EigenvalueIterationMonitor#monitor(double[], double[], int)
	 */
	public void monitor(double[] r, double[] eig, int i) {
		monitor(r, i);
	}

	/**
	 * @see jmp.iter.eig.EigenvalueIterationMonitor#monitor(double[], int)
	 */
	public void monitor(double[] r, int i) {
		array.add(i, r);
	}

}
