/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.eig;

import jmp.Matrix;
import jmp.Vector;

/**
 * Transforms the spectrum of the eigenvalue problem.
 */
public interface EigenvalueTransformation {

	/**
	 * Applies the transformation to x, store in y
	 * @return y
	 */
	Vector apply(Matrix A, Vector x, Vector y);

	/**
	 * Sets the shift
	 */
	void setShift(double sigma);

	/**
	 * Gets the shift
	 */
	double getShift();

	/**
	 * Unapplies the transformation to get the eigenvalue
	 */
	double eigenvalue(double e);

	/**
	 * Unapplies the transformation to get the eigenvalues
	 */
	double[] eigenvalue(double[] e);

}
