/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.eig;

import jmp.Matrix;
import jmp.Vector;

/**
 * Shifts the operator
 */
public class ShiftEigenvalueTransformation
	extends AbstractEigenvalueTransformation
	implements EigenvalueTransformation {

	protected double sigma;

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#apply(jmp.Matrix, jmp.Vector, jmp.Vector)
	 */
	public Vector apply(Matrix A, Vector x, Vector y) {
		return BLAS.add(sigma, x, BLAS.mult(A, x, y));
	}

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#setShift(double)
	 */
	public void setShift(double sigma) {
		this.sigma = sigma;
	}

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#getShift()
	 */
	public double getShift() {
		return sigma;
	}

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#eigenvalue(double)
	 */
	public double eigenvalue(double e) {
		return e - sigma;
	}

}
