/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.eig;

import jmp.Matrix;
import jmp.Vector;
import jmp.iter.linear.GMRES;
import jmp.iter.linear.LinearSolver;

/**
 * Shifts and inverts the operator
 */
public class ShiftInvertEigenvalueTransformation
	extends ShiftEigenvalueTransformation
	implements EigenvalueTransformation {

	protected double sigma, sigmaOld;

	protected Matrix Am;

	protected LinearSolver solver;

	/**
	 * Constructor for ShiftInvertEigenvalueTransformation. The matrix Am
	 * must be of the same size as A, and it is overwritten.
	 */
	public ShiftInvertEigenvalueTransformation(Matrix Am, Matrix A) {
		super();
		this.Am = BLAS.copy(A, Am);
		solver = new GMRES();
	}

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#apply(jmp.Matrix, jmp.Vector, jmp.Vector)
	 */
	public Vector apply(Matrix A, Vector x, Vector y) {
		Am = BLAS.addDiagonal(sigmaOld - sigma, Am);
		return solver.solve(Am, x, y);
	}

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#eigenvalue(double)
	 */
	public double eigenvalue(double e) {
		return sigma + 1. / e;
	}

	/**
	 * Sets the linear solver to use. Default is GMRES.
	 */
	public void setSolver(LinearSolver solver) {
		this.solver = solver;
	}

	/**
	 * @see jmp.iter.eig.EigenvalueTransformation#setShift(double)
	 */
	public void setShift(double sigma) {
		sigmaOld = this.sigma;
		super.setShift(sigma);
	}

}
