/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import jmp.BLAS;
import jmp.State;
import jmp.Vector;
import jmp.iter.AbstractIteration;
import jmp.iter.NotConvergedException;

/**
 * Partial implementation of a linear iteration object.
 */
public abstract class AbstractLinearIteration
	extends AbstractIteration
	implements LinearIteration {

	/**
	 * Local reference
	 */
	protected BLAS BLAS;

	/**
	 * Vector-norm
	 */
	protected int normType;

	/**
	 * Iteration monitor
	 */
	protected LinearIterationMonitor monitor;

	/**
	 * Constructor for AbstractLinearIteration. Default norm is the 2-norm with
	 * no monitoring.
	 */
	public AbstractLinearIteration() {
		super();
		this.BLAS = State.BLAS;
		normType = jmp.BLAS.NORM2;
		monitor = new NullLinearIterationMonitor();
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#converged(jmp.Vector, jmp.Vector)
	 */
	public boolean converged(Vector r, Vector x) throws NotConvergedException {
		return converged(BLAS.norm(r, normType), x);
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#converged(double, jmp.Vector)
	 */
	public boolean converged(double r, Vector x) throws NotConvergedException {
		monitor.monitor(r, x, iter);
		return convergedI(r, x);
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#converged(double)
	 */
	public boolean converged(double r) throws NotConvergedException {
		monitor.monitor(r, iter);
		return convergedI(r);
	}

	protected abstract boolean convergedI(double r, Vector x)
		throws NotConvergedException;

	protected abstract boolean convergedI(double r)
		throws NotConvergedException;

	/**
	 * @see jmp.iter.linear.LinearIteration#converged(jmp.Vector)
	 */
	public boolean converged(Vector r) throws NotConvergedException {
		return converged(BLAS.norm(r, normType));
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#getNormType()
	 */
	public int getNormType() {
		return normType;
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#setNormType(int)
	 */
	public void setNormType(int normType) {
		this.normType = normType;
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#getIterationMonitor()
	 */
	public LinearIterationMonitor getIterationMonitor() {
		return monitor;
	}

	/**
	 * @see jmp.iter.linear.LinearIteration#setIterationMonitor(jmp.iter.linear.LinearIterationMonitor)
	 */
	public void setIterationMonitor(LinearIterationMonitor monitor) {
		this.monitor = monitor;
	}

}
