/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import jmp.BLAS;
import jmp.Matrix;
import jmp.Vector;

/**
 * Partial implementation of LinearSolver
 */
public abstract class AbstractLinearSolver implements LinearSolver {

	/**
	 * Preconditioner to use
	 */
	protected Preconditioner M;

	/**
	 * Iteration
	 */
	protected LinearIteration iter;

	/**
	 * Algebra operations
	 */
	protected BLAS BLAS;

	/**
	 * Constructor for AbstractLinearSolver, using the DefaultConvergenceTest and
	 * no preconditioning.
	 */
	public AbstractLinearSolver() {
		M = new IdentityPreconditioner();
		iter = new DefaultLinearIteration();

		BLAS = jmp.State.BLAS;
	}

	/**
	 * @see jmp.iter.linear.LinearSolver#setPreconditioner(jmp.iter.linear.Preconditioner)
	 */
	public void setPreconditioner(Preconditioner M) {
		this.M = M;
	}

	/**
	 * @see jmp.iter.linear.LinearSolver#getPreconditioner()
	 */
	public Preconditioner getPreconditioner() {
		return M;
	}

	/**
	 * @see jmp.iter.linear.LinearSolver#getIteration()
	 */
	public LinearIteration getIteration() {
		return iter;
	}

	/**
	 * @see jmp.iter.linear.LinearSolver#setIteration(jmp.iter.linear.LinearIteration)
	 */
	public void setIteration(LinearIteration iter) {
		this.iter = iter;
	}

	/**
	 * Checks sizes of input data
	 */
	protected void checkSizes(Matrix A, Vector b, Vector x) {
		if (!A.isSquare())
			throw new ArrayIndexOutOfBoundsException("!A.isSquare()");
		if (b.size() != A.numRows())
			throw new ArrayIndexOutOfBoundsException("b.size() != A.numRows()");
		if (b.size() != x.size())
			throw new ArrayIndexOutOfBoundsException("b.size() != x.size()");
	}

	/**
	 * @see jmp.iter.linear.LinearSolver#solve(jmp.Matrix, jmp.Vector, jmp.Vector)
	 */
	public Vector solve(Matrix A, Vector b, Vector x)
		throws LinearNotConvergedException {
		checkSizes(A, b, x);
		solveI(A, b, x);
		return x;
	}

	/**
	 * Solves the system
	 */
	protected abstract void solveI(Matrix A, Vector b, Vector x)
		throws LinearNotConvergedException;

}
