/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import java.util.List;
import java.util.Vector;

/**
 * Stores all the residuals in an array, which the user can retrieve afterwards.
 */
public class ArrayLinearIterationMonitor implements LinearIterationMonitor {

	/**
	 * Contains all the residuals
	 */
	private List array;

	/**
	 * Constructor for ArrayLinearIterationMonitor.
	 */
	public ArrayLinearIterationMonitor() {
		array = new Vector();
	}

	/**
	 * Retrives all the residuals in an array. Array index is the iteration
	 * number.
	 * @return Array of residuals
	 */
	public double[] getArray() {
		Object[] arr = array.toArray();
		double[] ret = new double[arr.length];
		for (int i = 0; i < arr.length; ++i)
			ret[i] = ((Double) arr[i]).doubleValue();
		return ret;
	}

	/**
	 * @see jmp.iter.linear.LinearIterationMonitor#monitor(double, int)
	 */
	public void monitor(double r, int i) {
		array.add(i, new Double(r));
	}

	/**
	 * @see jmp.iter.linear.LinearIterationMonitor#monitor(double, jmp.Vector, int)
	 */
	public void monitor(double r, jmp.Vector x, int i) {
		array.add(i, new Double(r));
	}

}
