/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/*
 * Derived from public domain software at http://www.netlib.org/templates
 */

package jmp.iter.linear;

import jmp.Matrix;
import jmp.Vector;
import jmp.iter.Factory;

/**
 * Conjugate Gradients solver.
 * CG solves the symmetric positive definite linear system <code>Ax=b</code>
 * using the Conjugate Gradient method.
 * @author Templates
 */
public class CG extends AbstractLinearSolver implements LinearSolver {

	/**
	 * @see jmp.iter.linear.AbstractLinearSolver#solveI(jmp.Matrix, jmp.Vector, jmp.Vector)
	 */
	protected void solveI(Matrix A, Vector b, Vector x)
		throws LinearNotConvergedException {
		Vector[] temp = Factory.createVectors(b, 4);
		Vector p = temp[0], z = temp[1], q = temp[2], r = temp[3];

		double alpha = 0, beta = 0, rho = 0, rho_1 = 0;

		r = BLAS.multAdd(-1., A, x, b, r);

		for (iter.setFirst(); !iter.converged(r, x); iter.next()) {
			z = M.apply(A, r, z);
			rho = BLAS.dot(r, z);

			if (iter.isFirst())
				p = BLAS.copy(z, p);
			else {
				beta = rho / rho_1;
				p = BLAS.add(z, beta, p);
			}

			q = BLAS.mult(A, p, q);
			alpha = rho / BLAS.dot(p, q);

			x = BLAS.add(alpha, p, x);
			r = BLAS.add(-alpha, q, r);

			rho_1 = rho;
		}
	}

}
