/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/*
 * Derived from public domain software at http://www.netlib.org/templates
 */

package jmp.iter.linear;

import jmp.Matrix;
import jmp.Vector;
import jmp.iter.Factory;
import jmp.iter.NotConvergedException;

/**
 * Conjugate Gradients squared solver.
 * CGS solves the unsymmetric linear system <code>Ax = b</code> using the
 * Conjugate Gradient Squared method
 * @author Templates
 */
public class CGS extends AbstractLinearSolver implements LinearSolver {

	/**
	 * @see jmp.iter.linear.AbstractLinearSolver#solveI(jmp.Matrix, jmp.Vector, jmp.Vector)
	 */
	protected void solveI(Matrix A, Vector b, Vector x)
		throws LinearNotConvergedException {
		double rho_1 = 0, rho_2 = 0, alpha = 0, beta = 0;
		Vector[] temp = Factory.createVectors(b, 10);
		Vector p = temp[0],
			q = temp[1],
			u = temp[2],
			phat = temp[3],
			qhat = temp[4],
			vhat = temp[5],
			uhat = temp[6],
			sum = temp[7],
			r = temp[8],
			rtilde = temp[9];

		r = BLAS.multAdd(-1., A, x, b, r);
		rtilde = BLAS.copy(r, rtilde);

		for (iter.setFirst(); !iter.converged(r, x); iter.next()) {
			rho_1 = BLAS.dot(rtilde, r);
			if (rho_1 == 0)
				throw new LinearNotConvergedException(
					NotConvergedException.BREAKDOWN,
					iter.iterations(),
					"rho",
					BLAS.norm(r, iter.getNormType()));
			if (iter.isFirst()) {
				u = BLAS.copy(r, u);
				p = BLAS.copy(u, p);
			} else {
				beta = rho_1 / rho_2;
				u = BLAS.add(beta, q, r, u);
				sum = BLAS.add(beta, p, q, sum);
				p = BLAS.add(beta, sum, u, p); // p = u + beta*(beta*p+q)
			}

			phat = M.apply(A, p, phat);
			vhat = BLAS.mult(A, phat, vhat);
			alpha = rho_1 / BLAS.dot(rtilde, vhat);
			q = BLAS.add(-alpha, vhat, u, q);

			sum = BLAS.add(u, q, sum); // sum = u + q 
			uhat = M.apply(A, sum, uhat);
			x = BLAS.add(alpha, uhat, x);
			qhat = BLAS.mult(A, uhat, qhat);
			r = BLAS.add(-alpha, qhat, r);

			rho_2 = rho_1;
		}
	}

}
