/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/*
 * Derived from public domain software at http://www.netlib.org/templates
 */

package jmp.iter.linear;

import jmp.Matrix;
import jmp.Vector;
import jmp.iter.Factory;

/**
 * Iterative Refinement.
 * IR solves the unsymmetric linear system <code>Ax = b</code> using
 * Iterative Refinement (preconditioned Richardson iteration).
 * @author Templates
 */
public class IR extends AbstractLinearSolver implements LinearSolver {

	/**
	 * @see jmp.iter.linear.AbstractLinearSolver#solveI(jmp.Matrix, jmp.Vector, jmp.Vector)
	 */
	protected void solveI(Matrix A, Vector b, Vector x)
		throws LinearNotConvergedException {
		Vector[] temp = Factory.createVectors(b, 2);
		Vector z = temp[0], r = temp[1];

		r = BLAS.multAdd(-1., A, x, b, r);

		for (iter.setFirst(); !iter.converged(r, x); iter.next()) {
			z = M.apply(A, r, z);
			x = BLAS.add(z, x);
			r = BLAS.multAdd(-1., A, x, b, r);
		}
	}

}
