/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import jmp.Matrix;
import jmp.Vector;
import jmp.iter.NotConvergedException;

/**
 * Applies an iterative linear solver as preconditioner.
 * Only for transpose-free problems. The convergence criteria for the
 * preconditioner should not be too stringent as that can defeat the purpose of
 * the preconditioner.
 * <p>
 * Note that if the solver doesn't converge, the exceptions are caught
 * and promptly ignored.
 * </p>
 */
public class IterativeSolverPreconditioner
	extends AbstractPreconditioner
	implements Preconditioner {

	/**
	 * LinearSolver to use
	 */
	private LinearSolver preconditioner;

	/**
	 * Assigns the solver.
	 */
	public IterativeSolverPreconditioner(LinearSolver preconditioner) {
		this.preconditioner = preconditioner;
	}

	/**
	 * @see jmp.iter.linear.Preconditioner#apply(Matrix, Vector, Vector)
	 */
	public Vector apply(Matrix A, Vector b, Vector x) {
		try {
			return preconditioner.solve(A, b, x);
		} catch (NotConvergedException e) {
			// This is ignored. In fact, we probably don't want nor need
			// convergence here.
		}
		return x;
	}

	/**
	 * @see jmp.iter.linear.Preconditioner#transApply(Matrix, Vector, Vector)
	 */
	public Vector transApply(Matrix A, Vector b, Vector x) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see jmp.iter.linear.Preconditioner#setup(jmp.Matrix)
	 */
	public void setup(Matrix A) {
	}

}
