/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import jmp.Matrix;
import jmp.Vector;
import jmp.decomp.LUDecomposition;

/**
 * Complete LU factorization as preconditioner.
 * It should only be used to precondition blocks of the matrix, as for
 * instance in the BlockDiagonalPreconditioner. Transpose operation is not
 * supported.
 */
public class LUPreconditioner
	extends AbstractPreconditioner
	implements Preconditioner {

	/**
	 * The decomposition
	 */
	private LUDecomposition decomp;

	/**
	 * @see jmp.iter.linear.Preconditioner#apply(Matrix, Vector, Vector)
	 */
	public Vector apply(Matrix A, Vector b, Vector x) {
		double[] xs = decomp.solve(BLAS.getArrayCopy(b));
		return BLAS.setArray(xs, x);
	}

	/**
	 * @see jmp.iter.linear.Preconditioner#transApply(Matrix, Vector, Vector)
	 */
	public Vector transApply(Matrix A, Vector b, Vector x) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see jmp.iter.linear.Preconditioner#setup(jmp.Matrix)
	 */
	public void setup(Matrix A) {
		decomp = new LUDecomposition(BLAS.getArrayCopy(A));
	}

}
