/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import jmp.Vector;
import jmp.iter.Iteration;
import jmp.iter.NotConvergedException;

/**
 * Iteration for a linear system
 */
public interface LinearIteration extends Iteration {

	/**
	 * Checks for convergence
	 * @param r Residual-vector
	 * @param x State-vector
	 * @return True if converged
	 */
	boolean converged(Vector r, Vector x) throws NotConvergedException;

	/**
	 * Checks for convergence
	 * @param r Residual-norm
	 * @param x State-vector
	 * @return True if converged
	 */
	boolean converged(double r, Vector x) throws NotConvergedException;

	/**
	 * Checks for convergence
	 * @param r Residual-norm
	 * @return True if converged
	 */
	boolean converged(double r) throws NotConvergedException;

	/**
	 * Checks for convergence
	 * @param r Residual-vector
	 * @return True if converged
	 */
	boolean converged(Vector r) throws NotConvergedException;

	/**
	 * Sets new iteration monitor
	 */
	void setIterationMonitor(LinearIterationMonitor monitor);

	/**
	 * Returns current iteration monitor
	 */
	LinearIterationMonitor getIterationMonitor();

	/**
	 * Sets the vector-norm to calculate with
	 */
	void setNormType(int normType);

	/**
	 * Returns the vector-norm in use
	 */
	int getNormType();

}

