/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package jmp.iter.linear;

import jmp.iter.NotConvergedException;

/**
 * Exception for lack of convergence in a linear problem.
 * Contains the final computed residual.
 */
public class LinearNotConvergedException extends NotConvergedException {

	/**
	 * Final residual
	 */
	private double r;

	/**
	 * Constructor for LinearNotConvergedException.
	 * @param reason
	 * @param iterations
	 */
	public LinearNotConvergedException(int reason, int iterations, double r) {
		super(reason, iterations);
		this.r = r;
	}

	/**
	 * Constructor for LinearNotConvergedException.
	 * @param reason
	 * @param iterations
	 * @param message
	 */
	public LinearNotConvergedException(
		int reason,
		int iterations,
		String message, double r) {
		super(reason, iterations, message);
		this.r = r;
	}

	/**
	 * Returns final computed residual
	 */	
	public double getResidual() {
		return r;
	}

}
