/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.linear;

import jmp.Matrix;
import jmp.Vector;

/**
 * Krylov subspace linear solver.
 * Interface to a generic iterative Krylov based system solver. It solves the
 * linear problem <code>Ax=b</code> for x, using preconditioning and convergence
 * monitoring.
 */
public interface LinearSolver {

	/**
	 * Solves the given problem, writing result into the vector.
	 * @param A Matrix of the problem
	 * @param b Right hand side
	 * @param x Solution is stored here. Also uses as initial guess
	 * @return The solution vector x
	 */
	Vector solve(Matrix A, Vector b, Vector x) throws LinearNotConvergedException;

	/**
	 * Sets preconditioner
	 * @param M Preconditioner to use
	 */
	void setPreconditioner(Preconditioner M);

	/**
	 * Gets preconditioner
	 * @return Current preconditioner
	 */
	Preconditioner getPreconditioner();

	/**
	 * Sets iteration
	 * @param iter Iteration
	 */
	void setIteration(LinearIteration iter);

	/**
	 * Gets the iteration
	 * @return Current iteration
	 */
	LinearIteration getIteration();

}
