/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
* JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.iter.mixed;

import jmp.Matrix;
import jmp.Vector;
import jmp.iter.linear.LinearIteration;
import jmp.iter.linear.LinearNotConvergedException;
import jmp.iter.linear.LinearSolver;

/**
 * Solver for mixed problems
 */
public interface MixedSolver {

	/**
	 * Solves the given mixed problem.
	 */
	void solve(
		Matrix A,
		Matrix B,
		Matrix Bt,
		Matrix C,
		Vector q,
		Vector u,
		Vector f,
		Vector g)
		throws LinearNotConvergedException;

	/**
	 * Sets the solver to use internally
	 * @param solver LinearSolver to use
	 */
	void setSubSolver(LinearSolver solver);

	/**
	 * Returns internal solver
	 * @return The solver currently in use
	 */
	LinearSolver getSubSolver();

	/**
	 * Sets iteration
	 * @param iter Iteration
	 */
	void setIteration(LinearIteration iter);

	/**
	 * Gets the iteration
	 * @return Current iteration
	 */
	LinearIteration getIteration();

}
