/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jmp.test;

import java.util.Random;

import jmp.DenseVector;
import jmp.ElementalAccessVector;
import jmp.SparseVector;
import jmp.State;
import junit.framework.TestCase;

/**
 * Tests vector assembly
 */
public class VectorAssemblyTest extends TestCase {

	private static int nmax = 100, numax = 10, repeat = 3;

	/**
	 * Constructor for VectorAssemblyTest.
	 * @param arg0
	 */
	public VectorAssemblyTest(String arg0) {
		super(arg0);
	}

	private int getNu(int n, Random r) {
		return Math.min(n, Utilities.getInt(numax, r));
	}

	public void testDenseVector() {
		Random r = new Random();
		int n = Utilities.getInt(nmax, r), nu = getNu(n, r);
		ElementalAccessVector x = new DenseVector(n);
		repeatedAssembly(x, nu);
	}

	public void testSparseVector() {
		Random r = new Random();
		int n = Utilities.getInt(nmax, r), nu = getNu(n, r);
		ElementalAccessVector x = new SparseVector(n);
		repeatedAssembly(x, nu);
	}

	private void repeatedAssembly(ElementalAccessVector x, int nu) {
		for (int i = 0; i < repeat; ++i) {
			check(x, Utilities.setAssembleVector(x, nu));
			State.BLAS.zero(x);

			check(x, Utilities.setsAssembleVector(x, nu));
			State.BLAS.zero(x);

			check(x, Utilities.addAssembleVector(x, nu));
			State.BLAS.zero(x);

			check(x, Utilities.addsAssembleVector(x, nu));
			State.BLAS.zero(x);
		}
	}

	private void check(ElementalAccessVector x, double[] real) {
		checkEqual(Utilities.getElVector(x), real);
		checkEqual(Utilities.getVector(x), real);
		checkEqual(Utilities.getVectorCopy(x), real);
	}

	private void checkEqual(double[] A, double[] B) {
		assertTrue(A.length == B.length);
		for (int i = 0; i < A.length; ++i)
			assertEquals(B[i], A[i], 1e-12);
	}

}
