/*
 * Copyright (C) 2003 Bjrn-Ove Heimsund
 * 
 * This file is part of JMP.
 * 
 * JMP is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * JMP is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JMP; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/*
 * Derived from the public domain software package JAMA
 */

package jmp.util;

/**
 * Miscellaneous mathematical methods. This class cannot be instantiated.
 */
public final class Maths {

	/**
	 * No need to create an instance.
	 */
	private Maths() {
	}

	/**
	 * Computes <code>sqrt(a<sup>2</sup> + b<sup>2</sup>)</code> without
	 * underflow/overlow.
	 */
	public static double hypot(double a, double b) {
		double r;
		if (Math.abs(a) > Math.abs(b)) {
			r = b / a;
			r = Math.abs(a) * Math.sqrt(1 + r * r);
		} else if (b != 0) {
			r = a / b;
			r = Math.abs(b) * Math.sqrt(1 + r * r);
		} else
			r = 0.;
		return r;
	}

	/**
	 * Returns true if x is approximativly zero, else false
	 */
	public static boolean isZero(double x) {
		return Math.abs(x) < 1e-12;
	}

	/**
	 * Returns true if x is approximativly equal y, else false
	 */
	public static boolean equal(double x, double y) {
		return Math.abs(x - y) < 1e-12;
	}

}
